/*
 * Copyright 2022 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WSOLVER_H
#define WSOLVER_H

#include "heat.h"
#include "wObject.h"
#include "wTimer.h"
#include "wTimers.h"
#include "wElement.h"
#include <iostream>
#include <vector>
#include <memory>
#include <Eigen/Sparse>

using namespace tbox;

namespace heat
{

/**
 * @brief heat solver
 */

class HEAT_API Solver : public fwk::wSharedObject
{
public:
    int verbose;
#ifndef SWIG
    DisplayHook *dhook;

#endif
    std::shared_ptr<Problem> pbl;
    std::shared_ptr<tbox::LinearSolver> linsol; ///< linear solver

    bool stopit;            ///< stop the calculation
    int nthreads;           ///< # of threads for the assembly
    fwk::Timers tms;        ///< timers
    std::vector<double> T1; ///< nodal temperatures
    double restol;          ///< tolerance on the residual
    int itmax;              ///< max number of iterations
    bool save;              ///< calculate all results
public:
    Solver(std::shared_ptr<Problem> _pbl, std::shared_ptr<tbox::LinearSolver> _linsol);
    virtual ~Solver();

    virtual void start(MshExport *mshWriter);

    void setGUI(DisplayHook &hook) { dhook = &hook; }

    void stop() { stopit = true; }
#ifndef SWIG
    virtual void write(std::ostream &out) const override;
#endif

private:
    void buildK(Eigen::SparseMatrix<double, Eigen::RowMajor> &K2, Eigen::Map<Eigen::VectorXd> &rhs);
    void builds(Eigen::Map<Eigen::VectorXd> &s);
    void buildq(Eigen::Map<Eigen::VectorXd> &s);
    void buildqint(Eigen::VectorXd &qint);
};

} // namespace heat

#endif // WSOLVER_H
